<div class="reveal">
    <div class="slides">
        {% for section in show.sections %}
        <section>

            {% for slide in section.slides %}
            <!-- Slide {{ slide }} -->
            <section id="{{ slide }}">
                {{ show.do_markup(slide) | safe }}
            </section>
            {% endfor %}

        </section>
        {% endfor %}
    </div>
</div>
